﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

[System.Serializable]
public struct BSpline {
	[SerializeField]
	private List<Vector3> points;
//	private Vector3[] points;
	[SerializeField]
	private bool loopy;

	public bool Loop{
		get{
			return loopy;
		}
		set{
			loopy = value;
			if(value){
				points = new List<Vector3>(looping(points.ToArray()));
			}else{
				points = new List<Vector3>(unlooping(points.ToArray()));
			}
		}
	}

	public List<Vector3> Points{
		set{
			if(loopy){
				points = new List<Vector3>(looping(value.ToArray()));
			}else{
				points = value;
			}
		}
		get{
			if(loopy){
				return new List<Vector3>(unlooping(points.ToArray()));
			}else{
				return points;
			}
		}
	}

	public float length{
		get{
			return lenght(1f);
		}
	}

	public float lenght(float t, float precision = 0.1f){
		precision = Mathf.Clamp(precision, 0.001f, 1f);
		float l = 0f;
		Vector3 previous = points[0];
		for(float tee = precision ; tee <= t; tee+=precision){
			Vector3 current = onCurve(tee);
			l += Vector3.Distance(previous, current);
			previous = current;
		}
		return l;
	}

	public Vector3 onCurve(float t){
		t = Mathf.Clamp01(t);
		Vector3[] Derivation = points.ToArray();
		while(Derivation.Length > 1){
			Vector3[] nextDerivation = new Vector3[Derivation.Length - 1];
			for(int i = 0; i < nextDerivation.Length; i++){
				nextDerivation[i] = Vector3.Lerp(Derivation[i], Derivation[i+1], t);
			}
			Derivation = nextDerivation;
		}
		return Derivation[0];
	}



	public void trace(float TracingStep, Color Couleur, float duration = 0){
		Vector3 B = points[0];
		Vector3 BminusOne;

		TracingStep = Mathf.Min(1f,TracingStep);
		TracingStep = Mathf.Max(0.0001f,TracingStep);

		for(float t = 0; t <= 1f; t += TracingStep){
			BminusOne = B;
			B = onCurve(t);
			//Debug.Log("lining");
			Debug.DrawLine(BminusOne, B, Couleur, duration);
			//Gizmos.DrawLine(BminusOne, B);
		}
	}

	public void append(Vector3 endpoint){
		if(loopy){
			points.RemoveAt(points.Count - 1);
			points[points.Count - 1] = endpoint;
			points = new List<Vector3>(looping(points.ToArray()));
		}else{
			points.Add(endpoint);
		}
	}

	public void append(Vector3[] appendees){
		if(loopy){
			points.RemoveAt(points.Count - 1);
			points.AddRange(appendees);
			points = new List<Vector3>(looping(points.ToArray()));
		}else{
			points.AddRange(appendees);
			
		}
	}

	public void appendAt(Vector3 point, int index){
		if(loopy && index >= points.Count - 2 )
		{
			points.RemoveAt(points.Count - 1);
			points[points.Count - 1] = point;
			points = new List<Vector3>(looping(points.ToArray()));
		}else{
			points.Insert(index, point);
		}
	}

	public BSpline(Vector3 InPt, Vector3 InTgt, Vector3 OutPt, Vector3 OutTgt, bool loop = false){
		this.points = new List<Vector3>(new Vector3[]{InPt, InTgt, OutPt, OutTgt});
		this.loopy = loop;
	}

	public BSpline(Vector3 InPt, Vector3 InTgt, bool loop = false){
		this.points = new List<Vector3>(new Vector3[]{InPt, InTgt});
		this.loopy = loop;
	}

	public BSpline(Vector3[] pointsV, bool loop = false){
		this.loopy = loop;
		this.points = new List<Vector3>(pointsV);
		if(loop){
			this.points = new List<Vector3>(looping(points.ToArray()));
		}
	}

	public BSpline(List<Vector3> PointsList, bool loop = false){
		this.loopy = loop;
		this.points = PointsList;
		if(loop){
			this.points = new List<Vector3>(looping(points.ToArray()));
		}
	}

	private Vector3[] looping(Vector3[] inV){
		Vector3[] loopingPoints = new Vector3[inV.Length + 2];
		System.Array.Copy(inV, loopingPoints, inV.Length);
		loopingPoints[loopingPoints.Length - 1] = inV[0];
		loopingPoints[loopingPoints.Length - 2] = inV[0] - inV[1];
		return loopingPoints;
	}

	private Vector3[] unlooping(Vector3[] inV){
		System.Array.Resize(ref inV, inV.Length - 2);
		return inV;
	}

	public void delete(int index){
		points.RemoveAt(index);
	}
}
